require 'soar_idm/soar_idm'
require 'soar/registry/staff/translator/default'
require 'soar/registry/staff/directory/dynamo_db'
require 'soar/registry/staff/directory/stub'

module Soar
  module Registry
    module Staff
      module Identity
        class Base < SoarIdm::IdmApi

          attr_reader :directory
          attr_reader :translator
          attr_reader :client

          PRIMARY_KEY = 'identity_id'

          ##
          # @param [Hash] configuration
          #   for example see config/config.yml
          ##
          def initialize(configuration)
            @translator = configuration.key?('translator') ? Object.const_get(configuration['translator']['class']).new : Soar::Registry::Staff::Translator::Default.new
            @directory = Object::const_get(configuration['provider']['class']).new
            @directory.bootstrap(configuration['provider']['config'])
            @directory.authenticate(configuration['provider']['credentials'])
            @client = @directory.connect
            raise Soar::Registry::Staff::Directory::Error::BootstrapError if not @directory.bootstrapped? 
            raise Soar::Registry::Staff::Directory::Error::ConnectionError if not @directory.connected?
            raise Soar::Registry::Staff::Directory::Error::NotReadyError if not @directory.ready?
          end

          ##
          # @param [Hash] identity
          # @return [Array] list of roles
          def calculate_roles(identity)
            entry = @directory.fetch_identity(identity[PRIMARY_KEY])
            return nil if not entry
            identity = @translator.get_identity(entry)
            roles = []
            identity['roles'].each do |role, attributes|
              roles << role
            end
            roles
          end

          ##
          # @param [Hash] identity
          # @return [Array] list of identifiers
          ##
          def calculate_identifiers(identity)
            indexes = @directory.indexed_attributes
            entry = @directory.fetch_identity(identity[PRIMARY_KEY])
            identity = @translator.get_identity(entry)
            identifiers = []
            indexes.each { |index| 
              identifiers << identity[index]
            }
            identifiers
          end

          ## 
          # @param [Hash] identity
          # @param [String] role
          # @return [Hash] A hash of attributes 
          def calculate_attributes(identity, role)
            entry = @directory.fetch_identity(identity[PRIMARY_KEY])
            return nil if not entry
            identity = @translator.get_identity(entry)
            { role => identity['roles'][role] }
          end 

          ##
          # @param [Hash] identity
          # @return [Hash] Hash of attributes keyed by role
          def calculate_all_attributes(identity)
            entry = @directory.fetch_identity(identity[PRIMARY_KEY])
            @translator.get_identity(entry)
          end

        end
      end
    end
  end
end
