require 'soar/registry/staff/directory/dynamo_db'
require 'soar/registry/staff/test/directory/provider/base'
require 'hashie'
require 'yaml'

module Soar
  module Registry
    module Staff
      module Test
        module Directory
          module Provider
            class DynamoDb < Base

              def initialize(config={}, data_structure={}, data={})
                @config = config
                @identities = data
                @data_structure = data_structure
                @directory = Soar::Registry::Staff::Directory::DynamoDb.new
              end

              def create_data_structure
                if not @directory.client.list_tables.table_names.include?(valid_configuration['provider']['config']['table']) 
                  @directory.client.create_table(@data_structure)
                end
              end

              def delete_data_structure
                @directory.client.delete_table({
                  table_name: valid_configuration['provider']['config']['table']
                })
              end

              def load_sample_data
                @identities.each { |identity| 
                  @directory.client.put_item({
                    table_name: valid_configuration['provider']['config']['table'],
                    item: Hashie.symbolize_keys(deep_copy(identity))
                  })
                }
              end

              def connection_error
                configuration = @directory.instance_variable_get(:@configuration)
                configuration['endpoint'] = 'missing'
                @directory.instance_variable_set(:@configuration, configuration)
              end

              def networking_error
                configuration = @directory.instance_variable_get(:@configuration)
                configuration['endpoint'] = 'http://missing'
                @directory.instance_variable_set(:@configuration, configuration)
              end

              def valid_configuration?(config)
                @directory.configuration['region'] == config['provider']['config']['region'] and @directory.configuration['endpoint'] == config['provider']['config']['endpoint']
              end

              def invalid_configuration(valid_config)
                invalid_config = []
                ['table', 'region', 'endpoint'].each { |index| 
                  config = deep_copy(valid_config)
                  config['provider']['config'].delete(index)
                  invalid_config << config
                }
                return invalid_config
              end

              def valid_credentials?(credentials)
                @directory.credentials['access_key_id'] == credentials['username'] and @directory.credentials['secret_access_key'] == credentials['password']
              end

            end
          end
        end
      end
    end
  end
end
