require 'soar/registry/staff/directory/stub'
require 'soar/registry/staff/test/directory/provider/base'
require 'hashie'
require 'yaml'

module Soar
  module Registry
    module Staff
      module Test
        module Directory
          module Provider
            class Stub < Base

              attr_reader :directory

              def initialize(config={}, data_structure={}, data={})
                @config = config
                @identities = data
                @data_structure = data_structure
                @directory = Soar::Registry::Staff::Directory::Stub.new
              end

              def create_data_structure
                @directory.interface = Mince::HashyDb::Interface
              end

              def delete_data_structure
                @directory.interface.clear
              end

              def load_sample_data
                @identities.each { |identity| 
                  @directory.interface.add('identities', identity)
                }
              end

              def connection_error
                @directory.connection = false
              end

              def networking_error
                @directory.network = false
              end

              def valid_configuration
                @config['provider']['adaptor'] = "Soar::Registry::Staff::Directory::Stub"
                return @config
              end

              def valid_configuration?(config)
                config['provider']['adaptor'] == "Soar::Registry::Staff::Directory::Stub"
              end

              def valid_credentials?(credentials)
                true
              end

              def invalid_configuration(valid_config)
                invalid_config = deep_copy(valid_config)
                invalid_config['provider']['config'] = {}
                [invalid_config]
              end

            end
          end
        end
      end
    end
  end
end
